@@ -1,4 +1,7 @@
 //! Generates dynamically resolved identifiers.
+//!
+//! Common types are exposed at the crate level.
+//! This module provides additional low-level resource types.
 
 use core::fmt;
 use std::any::{type_name, TypeId};
@@ -6,10 +9,11 @@ use std::collections::BTreeMap;
 use std::hash::Hash;
 use std::marker::PhantomData;
 
+use getset::{CopyGetters, Getters, MutGetters};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
-use crate::{AbstractError, Context, Schema, Xylem};
+use crate::{AbstractError, Context, NoArgs, Schema, Xylem};
 
 /// An identifier for type `X`.
 ///
@@ -284,6 +288,7 @@ pub struct IdArgs {
 }
 
 /// Retrieves the original string ID for an identifiable object.
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct IdString<S: Schema, X: Identifiable<S>> {
     value: String,
     _ph:   PhantomData<&'static (S, X)>,
@@ -300,17 +305,12 @@ impl<S: Schema, X: Identifiable<S>> fmt::Debug for IdString<S, X> {
 }
 
 impl<S: Schema, X: Identifiable<S>> Clone for IdString<S, X> {
-    fn clone(&self) -> Self {
-        Self {
-            value: self.value.clone(),
-            _ph:   PhantomData,
-        }
-    }
+    fn clone(&self) -> Self { Self { value: self.value.clone(), _ph: PhantomData } }
 }
 
 impl<S: Schema, X: Identifiable<S>> Xylem<S> for IdString<S, X> {
     type From = ();
-    type Args = ();
+    type Args = NoArgs;
 
     #[inline]
     fn convert_impl(
@@ -342,18 +342,32 @@ impl<X: 'static> Default for IdCounter<X> {
 }
 
 /// Tracks the current ID.
-struct CurrentId {
+#[derive(Getters, CopyGetters)]
+pub struct CurrentId {
     /// The index of the current identifier.
     ///
     /// This does not use the `Id` type to avoid type parameters.
+    #[getset(get_copy = "pub")]
     id:     usize,
     /// The type ID of the parent.
+    #[getset(get_copy = "pub")]
     parent: TypeId,
     /// The original string ID.
+    #[getset(get = "pub")]
     string: String,
 }
 
-struct GlobalIdStore<S: Schema, X: Identifiable<S>> {
+/// Stores the globally tracked IDs.
+///
+/// This is a low-level implementation.
+/// Prefer using [`IdArgs`] with `track` and `import` for a more simple and stable API.
+#[derive(Getters, MutGetters)]
+pub struct GlobalIdStore<S: Schema, X: Identifiable<S>> {
+    /// The actual storage.
+    ///
+    /// The key is the ID path to the scope `X::Scope`,
+    /// and the value is a list of the IDs in that scope.
+    #[getset(get = "pub", get_mut = "pub")]
     ids: BTreeMap<Vec<usize>, Vec<String>>,
     _ph: PhantomData<&'static (S, X)>,
 }
